#!/usr/bin/python

'''
This script estimates the number of anchor points that should be calculate for each protein and for the density map
'''
import sys
import IMP
import IMP.atom
import math

from assembly_input_reader import *
class AnchorPointEstimator:
    def __init__(self,asmb_data):
        self.asmb_data=asmb_data
        self.prot_num_ap=[]
        self.prot_num_res=[]
        self.total_ap=0
    def estimate(self,res4ap):
        #find the number of residue of each protein
        mhs=IMP.atom.Hierarchies()
        sel=IMP.atom.CAlphaPDBSelector()
        mdl=IMP.Model()
        total_ap=0
        for p in self.asmb_data.proteins:
            mhs.append(IMP.atom.read_pdb(p.prot_fn,mdl,sel))
            size=len(IMP.core.get_leaves(mhs[-1]))
            self.prot_num_res.append(size)
            if size<res4ap:
                n=1
            else:
                n=size/res4ap
                #if math.fmod(size,res4ap) > res4ap/2:
                if math.fmod(size,res4ap) > 0:
                    n=n+1
            self.prot_num_ap.append(n)
            self.total_ap = self.total_ap + n

def usage():
    if len(sys.argv) != 3:
        print sys.argv[0] + " <assembly.input> <residue for anchor point>"
        sys.exit()
def run():
    asmb_data = AssemblyReader(sys.argv[1])
    res4ap=int(sys.argv[2])
    ape=AnchorPointEstimator(asmb_data)
    ape.estimate(res4ap)
    total=0
    for i,p in enumerate(asmb_data.proteins):
        print p.prot_fn , "  " ,str(ape.prot_num_res[i])," ",str(ape.prot_num_ap[i])
    print asmb_data.map.filename ," ",ape.total_ap
if __name__=="__main__":
    usage()
    run()
