import os
import IMP.atom

'''
if coarse >0 then we apply coarsing of the data
'''
def read_components(asmb,mdl):
    mhs = IMP.atom.Hierarchies()
    #sel = IMP.atom.CAlphaPDBSelector()
    sel = IMP.atom.NonWaterPDBSelector()
    for p in asmb.proteins:
        mh=IMP.atom.read_pdb(p.prot_fn,
                             mdl,sel)
        IMP.atom.add_radii(mh)
        mhs.append(mh)
    return mhs

def read_ref_components(asmb,mdl):
    mhs = IMP.atom.Hierarchies()
    #sel = IMP.atom.CAlphaPDBSelector()
    sel = IMP.atom.NonWaterPDBSelector()
    for p in asmb.proteins:
        mh=IMP.atom.read_pdb(p.ref_fn,mdl,sel)
        IMP.atom.add_radii(mh)
        mhs.append(mh)
    return mhs

class DensityHeader:
    def __init__(self):
        self.filename=""
        self.resolution=0.
        self.spacing=0.
        self.x_origin=0.
        self.y_origin=0.
        self.z_origin=0.
        self.pdb_fine_ap_fn="" #for fitting
        self.pdb_coarse_ap_fn="" #for mapping
        self.cmm_ap_fn=""

    def suggest_ap_cmm_fn(self):
        if self.cmm_ap_fn=="":
            return os.getcwd()+"/"+os.path.basename(self.filename).split(".mrc")[0]+"_anchor_points.cmm"
        return self.cmm_ap_fn
    def suggest_coarse_ap_pdb_fn(self):
        if self.pdb_coarse_ap_fn=="":
            return os.getcwd()+"/"+os.path.basename(self.filename).split(".mrc")[0]+"_coarse_anchor_points.pdb"
        return self.pdb_coarse_ap_fn

    def suggest_fine_ap_pdb_fn(self):
        if self.pdb_fine_ap_fn=="":
            return os.getcwd()+"/"+os.path.basename(self.filename).split(".mrc")[0]+"_coarse_anchor_points.pdb"
        return self.pdb_fine_ap_fn

class ProteinHeader:
    def __init__(self):
        self.name="name"
        self.prot_fn=""
        self.pdb_ap_fn=""
        self.num_ap=0
        self.transformations_fn=""
        self.cmm_ap_fn=""
        self.ref_fn=""
    def suggest_ap_cmm_fn(self):
        if self.cmm_ap_fn=="":
            return os.getcwd()+"/"+os.path.basename(self.prot_fn).split(".pdb")[0]+"_anchor_points.cmm"
        return self.cmm_ap_fn
    def suggest_ap_pdb_fn(self):
        if self.pdb_ap_fn=="":
            return os.getcwd()+"/"+os.path.basename(self.prot_fn).split(".pdb")[0]+"_anchor_points.pdb"
        return self.pdb_ap_fn

class AssemblyReader:
    def __init__(self,assembly_filename):
        self.assembly_filename = assembly_filename
        self.proteins=[]
        self.map=DensityHeader()
        self.read()
    def read(self):
        f = open(self.assembly_filename)
        lines = f.readlines()
        f.close()
        #first line in header
        li=1
        while li<len(lines):
            s=lines[li].split("|")
            if len(s)==7:
                self.proteins.append(ProteinHeader())
                self.proteins[-1].name=os.path.expandvars(s[0])
                self.proteins[-1].prot_fn=os.path.expandvars(s[1])
                self.proteins[-1].pdb_ap_fn=os.path.expandvars(s[2])
                self.proteins[-1].num_ap=int(os.path.expandvars(s[3]))
                self.proteins[-1].transformations_fn=os.path.expandvars(s[4])
                self.proteins[-1].ref_fn=os.path.expandvars(s[5])
                li=li+1
            else:
                break
        #map header
        li=li+1
        #map line
        s=lines[li].split("|")
        self.map.filename=os.path.expandvars(s[0])
        self.map.resolution=float(s[1])
        self.map.spacing=float(s[2])
        self.map.x_origin=float(s[3])
        self.map.y_origin=float(s[4])
        self.map.z_origin=float(s[5])
        self.map.pdb_fine_ap_fn=os.path.expandvars(s[6])
        self.map.pdb_coarse_ap_fn=os.path.expandvars(s[7])

    def write(self,output_fn):
        f = open(output_fn,"w")
        while li<len(lines):
            s=lines[li].split("|")
            if len(s)==6:
                self.proteins.append(ProteinHeader())
                self.proteins[-1].name=os.path.expandvars(s[0])
                self.proteins[-1].prot_fn=os.path.expandvars(s[1])
                self.proteins[-1].pdb_ap_fn=os.path.expandvars(s[2])
                self.proteins[-1].num_ap=os.path.expandvars(s[3])
                self.proteins[-1].transformations_fn=os.path.expandvars(s[4])
                self.proteins[-1].ref_fn=os.path.expandvars(s[5])
                li=li+1
            else:
                break
        #map header
        li=li+1
        #map line
        s=lines[li].split("|")
        self.map.filename=os.path.expandvars(s[0])
        self.map.resolution=float(s[1])
        self.map.spacing=float(s[2])
        self.map.x_origin=float(s[3])
        self.map.y_origin=float(s[4])
        self.map.z_origin=float(s[5])
        self.map.pdb_fine_ap_fn=os.path.expandvars(s[6])
        self.map.pdb_coarse_ap_fn=os.path.expandvars(s[7])

            
