#!/usr/bin/python

from run_domino import run_domino
import sys
import IMP
import IMP.atom
import time
import IMP.domino
from fitting_solutions_reader import read_transformations
from assembly_input_reader import AssemblyReader

def set_weighted_excluded_volume_restraint(rb1,rb2):
  IMP.multifit.create_weighted_excluded_volume_restraint(rb1,rb2)

def usage():
  if len(sys.argv) != 5:
    print sys.argv[0] + " <assembly.input> <comp1 index> <comp2 index> <output filename>"
    print "Calculate pairwise terms between comp1 and comp2"
    sys.exit()

def add_states(discrete_set,rb_p,ts):
    '''
    Add the states of the rigid body to the discrete sampling space
    ts - the discrete sampling space of the rigid body
    '''
    mdl=rb_p.get_model()
    for t in ts:
        state_p=IMP.Particle(mdl)
        IMP.domino.Transformation.setup_particle(state_p,t)
        discrete_set.add_mapped_state(rb_p,state_p)
    
def set_discrete_sampling_space(asmb,rbs,comp1,comp2):
    discrete_set =IMP.domino.TransformationMappedDiscreteSet(rbs)
    #add valid transformation for each rigid body
    ts1=read_transformations(asmb.proteins[comp1].transformations_fn)
    ts2=read_transformations(asmb.proteins[comp2].transformations_fn)
    add_states(discrete_set,rbs[0].get_particle(),ts1)
    add_states(discrete_set,rbs[1].get_particle(),ts2)
    return discrete_set


def run(assembly_fn,comp1_ind,comp2_ind,output_fn):
    IMP.set_log_level(IMP.SILENT)
    asmb = AssemblyReader(assembly_fn)
    if asmb.proteins[comp1_ind].name>asmb.proteins[comp2_ind].name:
      t=comp1_ind
      comp1_ind=comp2_ind
      comp2_ind=t
    mdl = IMP.Model()
    #mdl.set_is_incremental(True)
    #read components
    mhs = IMP.atom.Hierarchies()
    #sel = IMP.atom.NonWaterPDBSelector()
    sel = IMP.atom.CAlphaPDBSelector()
    mhs=IMP.atom.Hierarchies()
    print "calculating pairwise terms between: \n"+ \
           asmb.proteins[comp1_ind].prot_fn + " and " + \
           asmb.proteins[comp2_ind].prot_fn + "\n"
    mhs.append(IMP.atom.read_pdb(asmb.proteins[comp1_ind].prot_fn,mdl,sel))
    mhs.append(IMP.atom.read_pdb(asmb.proteins[comp2_ind].prot_fn,mdl,sel))
    IMP.atom.add_radii(mhs[0])
    IMP.atom.add_radii(mhs[1])
    mhs[0].get_particle().add_attribute(IMP.domino.node_name_key(),asmb.proteins[comp1_ind].name)
    mhs[1].get_particle().add_attribute(IMP.domino.node_name_key(),asmb.proteins[comp2_ind].name)
    #set rigid boides
    rbs=IMP.helper.set_rigid_bodies(mhs)
    #set the discrete sampling space
    discrete_set=set_discrete_sampling_space(asmb,rbs,comp1_ind,comp2_ind)
    #read sampling space
    tu=IMP.domino.TransformationUtils(rbs,True) #because we need the reference to be the initial positions
    #r_col_wrap=IMP.helper.create_simple_excluded_volume_on_rigid_bodies(rbs)
    set_weighted_excluded_volume_restraint(rbs[0],rbs[1])
    #--score
    f=open(output_fn,"w")
    rb1=rbs[0]
    rb2=rbs[1]
    rb1_p=rb1.get_particle()
    rb2_p=rb2.get_particle()
    #mdl.set_is_incremental(True)
    #fix rb1
    print "number of mapped states for prot1: " + str(discrete_set.get_number_of_mapped_states(rb1_p))
    print "number of mapped states for prot2: " + str(discrete_set.get_number_of_mapped_states(rb2_p))
    f.write(rb1_p.get_value(IMP.domino.node_name_key())+"|"+ \
            rb2_p.get_value(IMP.domino.node_name_key())+"|\n")
    for rb1_state in range(discrete_set.get_number_of_mapped_states(rb1_p)):
        tu.move2state(
            rb1_p,
            discrete_set.get_mapped_state(rb1_p,rb1_state))
        #fix rb2
        for rb2_state in range(discrete_set.get_number_of_mapped_states(rb2_p)):
            tu.move2state(
                rb2_p,
                discrete_set.get_mapped_state(rb2_p,rb2_state))
            score=mdl.evaluate(False)
            f.write(str(rb1_state)+"|"+str(rb2_state)+"|"+str(score)+"|\n")
    f.close()
if __name__ == '__main__':
  usage()
  IMP.set_log_level(IMP.SILENT)
  assembly_fn = sys.argv[1]
  comp1 = int(sys.argv[2])
  comp2 = int(sys.argv[3])
  scores_fn=sys.argv[4]
  run(assembly_fn,comp1,comp2,scores_fn)
