#!/usr/bin/python

from run_domino import run_domino
import sys
import IMP
import IMP.atom
import IMP.domino
from fitting_solutions_reader import read_transformations
from assembly_input_reader import AssemblyReader

def usage():
  if len(sys.argv) != 4:
    print sys.argv[0] + " <assembly.input> <comp1 index> <output filename>"
    print "Calculate singleton terms for comp1 "
    sys.exit()

def add_states(discrete_set,rb_p,ts):
    '''
    Add the states of the rigid body to the discrete sampling space
    ts - the discrete sampling space of the rigid body
    '''
    mdl=rb_p.get_model()
    for t in ts:
        state_p=IMP.Particle(mdl)
        IMP.domino.Transformation.setup_particle(state_p,t)
        discrete_set.add_mapped_state(rb_p,state_p)
    
def set_discrete_sampling_space(asmb,rbs,comp1):
    discrete_set =IMP.domino.TransformationMappedDiscreteSet(rbs)
    #add valid transformation for each rigid body
    ts1=read_transformations(asmb.proteins[comp1].transformations_fn)
    add_states(discrete_set,rbs[0].get_particle(),ts1)
    return discrete_set

def run(assembly_fn,comp1_ind,output_fn):
    asmb = AssemblyReader(assembly_fn)
    mdl = IMP.Model()
    #mdl.set_is_incremental(True)
    #read components
    mhs = IMP.atom.Hierarchies()
    sel = IMP.atom.CAlphaPDBSelector()
    mhs=IMP.atom.Hierarchies()
    mhs.append(IMP.atom.read_pdb(asmb.proteins[comp1_ind].prot_fn,mdl,sel))
    IMP.atom.add_radii(mhs[0])
    mhs[0].get_particle().add_attribute(IMP.domino.node_name_key(),asmb.proteins[comp1_ind].name)        
    #load the density map
    # dmap=IMP.helper.load_em_density_map(asmb.map.filename,asmb.map.spacing,asmb.map.resolution)
    # dmap.set_origin(IMP.algebra.Vector3D(asmb.map.x_origin,asmb.map.y_origin,asmb.map.z_origin))
    mrw=IMP.em.MRCReaderWriter()
    dmap=IMP.em.read_map(asmb.map.filename,mrw);
    dmap.update_voxel_size(asmb.map.spacing)
    dmap.get_header_writable().set_resolution(asmb.map.resolution)
    dmap.set_origin(IMP.algebra.Vector3D(asmb.map.x_origin,asmb.map.y_origin,asmb.map.z_origin))
    #set rigid boides
    rbs=IMP.helper.set_rigid_bodies(mhs)
    #set the discrete sampling psace
    discrete_set=set_discrete_sampling_space(asmb,rbs,comp1_ind)
    #read sampling space
    tu=IMP.domino.TransformationUtils(rbs,True) #because we need the reference to be the initial positions
    r_cc_wrap = IMP.helper.create_simple_em_fit(mhs,dmap)
    mdl.add_restraint(r_cc_wrap.get_restraint())
    f=open(output_fn,"w")
    rb1=rbs[0]
    rb1_p=rb1.get_particle()

    #fix rb1
    print "number of mapped states for prot1: " + str(discrete_set.get_number_of_mapped_states(rb1_p))
    f.write(rb1_p.get_value(IMP.domino.node_name_key())+"|\n")
    for rb1_state in range(discrete_set.get_number_of_mapped_states(rb1_p)):
        tu.move2state(
            rb1_p,
            discrete_set.get_mapped_state(rb1_p,rb1_state))
        score=mdl.evaluate(False)
        f.write(str(rb1_state)+"|"+str(score)+"|\n")
    f.close()
if __name__ == '__main__':
  usage()
  IMP.set_log_level(IMP.SILENT)
  assembly_fn = sys.argv[1]
  comp1 = int(sys.argv[2])
  run(assembly_fn,comp1,sys.argv[3])
