import IMP.algebra

class FittingSolutions:
    def __init__(self,sols):
        self.sols=sols
    def __len__(self):
        return len(self.sols)

    def __getitem__(self, index):
        return self.sols[index]

    '''
    Find all of the solutions that are close to the input point
    '''
    def find_close_solutions(self,fitted_mh,point,dist):
        mh_centeroid=IMP.core.get_centroid(IMP.core.XYZsTemp(IMP.core.get_leaves(fitted_mh)))
        dist2=dist*dist
        close_indexes=[]
        print "point : " + str(point)
        print "mh_centeroid : " + str(mh_centeroid)
        aa=[]
        for i,sol in enumerate(self.sols):
            v1=sol.transformation.get_transformed(mh_centeroid)
            aa.append(IMP.algebra.get_distance(v1,point))

            if IMP.algebra.get_squared_distance(v1,point) < dist2:
                close_indexes.append(i)
        print aa
        print close_indexes
        return close_indexes
        

class FittingRecord:
    def __init__(self,line):
        self.parse_line(line)
    def reset(self):
        self.index=0
        self.sol_fn=""
        self.transformation=None
        self.match_size=0
        self.match_avg_dist=0.
        self.fitting_score=0.
        self.rmsd2ref=0.
    def parse_line(self,l):
        s=l.split("|")
        self.index=int(s[0])
        self.sol_fn=s[1]
        s1=s[2].split()
        s2=s[4].split("(")[1].split(")")[0].split(",")
        rot = IMP.algebra.Rotation3D(float(s1[0]),float(s1[1]),float(s1[2]),
                                     float(s1[3]))
        t = IMP.algebra.Vector3D(float(s2[0]),float(s2[1]),float(s2[2]))
        self.transformation = IMP.algebra.Transformation3D(rot,t)
        self.match_size=int(s[5])
        self.match_avg_dist=float(s[6])
        self.fitting_score=float(s[7])
        self.rmsd2ref=float(s[8])

    def write_line(self):
        l=str([self.index,self.sol_fn]).replace(",","|")[1:-1].replace("'","")+"|"
        l=l+str(self.transformation.get_rotation().get_quaternion()).replace(","," ")[1:-1]+"||"
        l=l+str(self.transformation.get_translation())+"|"
        l=l+str([self.match_size,self.match_avg_dist,self.fitting_score,self.rmsd2ref]).replace(",","|")[1:-1]
        return l

def read_transformations(trans_fn):
  f = open(trans_fn)
  lines = f.readlines()
  f.close()
  transformations=[]
  for l in lines[1:]:
    fr=FittingRecord(l)
    transformations.append(fr.transformation)
  return transformations

def read_fitting_records(trans_fn):
  f = open(trans_fn)
  lines = f.readlines()
  f.close()
  transformations=[]
  for l in lines[1:]:
    fr=FittingRecord(l)
    transformations.append(fr)
  return transformations


def write_transformations(trans,trans_fn):
  f = open(trans_fn,"w")
  f.write(" solution index | solution filename | transformation  | match size | match average distance | fitting score  | RMSD to reference\n")
  for t in trans:
      f.write(t.write_line()+"\n")
  f.close()

def parse_fitting_file(fitting_fn):
  f = open(fitting_fn)
  lines = f.readlines()
  f.close()
  sols=[]
  for l in lines[1:]:
    sols.append(FittingRecord(l))
  return FittingSolutions(sols)
  
