import IMP.atom
import IMP.core
import IMP.helper
import IMP.multifit
import sys
'''
The function assumes that the leaves have mass attributes
'''
def coarse_molecule(mh,coarse_lvl):
    if coarse_lvl==0:
        return mh
    return IMP.helper.create_simplified_by_residue(mh,coarse_lvl)

def coarse_molecules(mhs,coarse_lvl):
    coarse_mhs=IMP.atom.Hierarchies()
    for mh in mhs:
        coarse_mhs.append(coarse_molecule(mh,coarse_lvl))
    return coarse_mhs

def set_rigid_bodies(mdl,mhs):
    rbs=IMP.core.RigidBodies()
    IMP.helper.set_rigid_bodies(mhs)
    for mh in mhs:
        rbs.append(IMP.core.RigidBody(mh.get_particle()))
    return rbs
def transform_mol(mh,trans):
    xyzs=IMP.core.XYZs(IMP.atom.get_leaves(mh))
    for xyz in xyzs:
        xyz.set_coordinates(trans.get_transformed(xyz.get_coordinates()))
def set_connectivity_restraint_between_rigid_bodies(rb1,rb2):
    ps=IMP.Particles()
    ps.append(rb1)
    ps.append(rb2)
    mdl=rb1.get_model()
    pss = IMP.core.SphereDistancePairScore(IMP.core.HarmonicUpperBound(0,1))
    evr = IMP.core.ConnectivityRestraint(pss)
    evr.set_particles(ps)
    mdl.add_restraint(evr)
    evr.evaluate(None)
    #mdl.update_score_states()
    return evr

def set_colision_detection_between_rigid_bodies(rb1,rb2):
    rbs=IMP.core.RigidBodies()
    rbs.append(rb1)
    rbs.append(rb2)
    simple_r = IMP.helper.create_simple_excluded_volume_on_rigid_bodies(rbs)
    r = simple_r.get_restraint()
    print r
    rb1.get_model().add_restraint(r)
    return r

    # mdl=rb1.get_model()
    # lsc= IMP.core.ListSingletonContainer()
    # lsc.add_particle(rb1)
    # lsc.add_particle(rb2)
    # #Set up the nonbonded list
    # nbl= IMP.core.ClosePairsScoreState(lsc)
    # #bacause you want to restraint the actual rigid bodies and not the its particles
    # nbl.set_close_pairs_finder(IMP.core.RigidClosePairsFinder())
    # mdl.add_score_state(nbl);
    # #Set the amount particles need to move before the list is updated
    # nbl.set_slack(2.0);
    # #Set up excluded volume
    # #std=3.5
    # #k = 1./(std*std)
    # pss = IMP.core.SphereDistancePairScore(IMP.core.HarmonicLowerBound(0,1))
    # evr = IMP.core.PairsRestraint(pss, nbl.get_close_pairs_container())
    # mdl.add_restraint(evr)
    # IMP.set_log_level(IMP.VERBOSE)
    # evr.evaluate(None)

def set_weighted_excluded_volume_between_rigid_bodies(rb1,rb2):
    return IMP.multifit.create_weighted_excluded_volume_restraint(rb1,rb2)

def load_map(map_fn,spacing,resolution,x,y,z):
    dmap=IMP.helper.load_em_density_map(map_fn,spacing,resolution)
    dmap.set_origin(x,y,z)
    return dmap

def set_em_restraint2(mhs,dmap):
    ps = IMP.Particles()
    for mh in mhs:
      ps=ps+IMP.core.get_leaves(mh)
    fit_rs = IMP.em.FitRestraint(ps,dmap,
                                 IMP.core.XYZR.get_default_radius_key(),
                                 IMP.atom.Mass.get_mass_key(),
                                 1.)
    #mh.get_particle().get_model().add_restraint(fit_rs)
    fit_rs.set_model(mh.get_particle().get_model())
    return fit_rs
def set_em_restraint(mh,dmap):

    try:
        xyz_p = IMP.core.get_leaves(mh)
        mdl=mh.get_particle().get_model()
    except:
        xyz_p=IMP.Particles()
        mdl=mh.get_particle().get_model()
        for m in mh:
            xyz_p = xyz_p+IMP.core.get_leaves(m)

    fit_rs = IMP.em.FitRestraint(xyz_p,dmap,
                                 IMP.core.XYZR.get_default_radius_key(),
                                 IMP.atom.Mass.get_mass_key(),
                                 1.)
    #mh.get_particle().get_model().add_restraint(fit_rs)
    fit_rs.set_model(mdl)
    return fit_rs
      
