import sys
import math
import IMP
import IMP.atom
import IMP.domino
import probstat

class Mapping:
  def __init__(self,asmb,mhs,aps,discrete_set,dist_t):
    self.asmb=asmb
    self.mhs=mhs
    self.aps=aps
    self.discrete_set = discrete_set
    self.dist_t=dist_t
    self.calc_num_of_states()

  def calc_num_of_states(self):
    self.num_trans=[]
    aps_vec=[]
    for ap in self.aps:
      aps_vec.append(IMP.core.XYZ(ap).get_coordinates())
    for mh_ind in range(len(self.asmb.proteins)):
      self.num_trans.append([])
      for ap_ind in range(len(self.aps)):
        self.num_trans[-1].append(0)
    for mh_ind,mh in enumerate(self.mhs):
      comp_cent=IMP.core.get_centroid(IMP.core.XYZsTemp(IMP.core.get_leaves(mh)))
      for state_ind in range(self.discrete_set.get_number_of_mapped_states(mh.get_particle())):
        state_p=self.discrete_set.get_mapped_state(mh.get_particle(),state_ind);
        t = IMP.domino.Transformation(state_p).get_transformation();
        t_comp_cent = t.get_transformed(comp_cent)
        for ap_ind,ap in enumerate(aps_vec):
          dist=IMP.algebra.get_distance(t_comp_cent,ap);
          #print "for mh_ind:",mh_ind," with state:",state_ind," ap_ind:",ap_ind, " the distance is: ",dist, "comp_cent: " , comp_cent[0],":",comp_cent[1]
          if (dist<self.dist_t):
            self.num_trans[mh_ind][ap_ind]=self.num_trans[mh_ind][ap_ind]+1
    for mh_ind in range(len(self.asmb.proteins)):
      l=""
      for ap_ind in range(len(self.aps)):
        l=l+str(self.num_trans[mh_ind][ap_ind])+"|"
      print l
  def build_mappings_from_components_to_anchor_points(self):
    cc=probstat.Permutation(range(len(self.asmb.proteins)))
    cc_pruned=[]
    #save the number of transformations for each protein mapped to each protein
    map_ind=0
    while map_ind < len(cc):
      if self.is_valid_mapping(cc[map_ind]):
        cc_pruned.append(cc[map_ind])
        map_ind=map_ind+1
      else:
        map_ind=self.next_index(map_ind,cc[map_ind])
    return cc_pruned
  def is_valid_mapping(self,mapping_comp2ap):
    for prot_ind,ap_ind in enumerate(mapping_comp2ap):
      if self.num_trans[prot_ind][ap_ind] == 0:
        return False
    print "The mapping : " , mapping_comp2ap, "is valid"
    return True

  def next_index(self,cur_ind,mapping_comp2ap):
    #find first invalid
    comp_ind=0
    found=False
    while (not found) and comp_ind<len(mapping_comp2ap):
      if self.num_trans[comp_ind][mapping_comp2ap[comp_ind]] != 0:
        comp_ind=comp_ind+1
      else:
        found=True
    if not found and comp_ind==len(mapping_comp2ap):
      print "Problem finding the next index"
      sys.exit(-1)
    if comp_ind  > len(mapping_comp2ap)-3: #one of the last 2 indexes
      return cur_ind+1
    #return cur_ind+math.factorial(len(mapping_comp2ap)-comp_ind-1)-1
    #Houston guys do not have factorial in their python
    f=1
    for i in range(1,len(mapping_comp2ap)-comp_ind):
      f=f*i
    return cur_ind+1
    return cur_ind+f-1
