#!/usr/bin/python

'''
This script estimates the number of anchor points that should be calculate for each protein and for the density map and runs anchor point detection
'''
import sys
import IMP
import IMP.atom
from assembly_input_reader import *
from anchor_point_estimator import *
def usage():
    if len(sys.argv) != 3:
        print sys.argv[0] + " <assembly.input> <em threshold>"
        sys.exit()

def gmm_pdb_command(prot_fn,num_ap,pdb_ap_fn,cmm_ap_fn=""):
    command = os.path.expandvars("/opt/multifit")+"/bin/gmm_pdb "+prot_fn+ " " + str(num_ap) + " " + pdb_ap_fn #+ " --backbone"
    if cmm_ap_fn != "":
        command = command + " --cmm " + cmm_ap_fn
    return command

def gmm_em_command(em_fn,num_ap,threshold, apix,ap_fn,x_orig,y_orig,z_orig,cmm_fn=""):
    command = os.path.expandvars("/opt/multifit")+"/bin/gmm_em "+ em_fn + " " + str(num_ap) + " " + str(threshold) + " " + ap_fn + " --x "+str(x_orig)+ " --y " + str(y_orig) + " --z " + str(z_orig) + " --apix " +  str(apix)
    if cmm_fn != "":
        command = command + " --cmm " + cmm_ap_fn
    return command

def trn_pdb_command(prot_fn,num_ap,pdb_ap_fn,cmm_ap_fn=""):
    command = os.path.expandvars("/opt/multifit")+"/bin/trn_pdb "+prot_fn+ " " + str(num_ap) + " " + pdb_ap_fn
    if cmm_ap_fn != "":
        command = command + " --cmm " + cmm_ap_fn
    return command

def trn_em_command(em_fn,num_ap,threshold, apix,ap_fn,cmm_fn=""):
    command = os.path.expandvars("/opt/multifit")+"/bin/trn_em "+ em_fn + " " + str(num_ap) + " " + str(threshold) + " " + ap_fn + " --apix " + str(apix)
    if cmm_fn != "":
        command = command + " --cmm " + cmm_ap_fn
    return command

def run():
    asmb_data = AssemblyReader(sys.argv[1])
    ape=AnchorPointEstimator(asmb_data)
    total_num_ap=0
    #ape.estimate(res4ap)
    for i,p in enumerate(asmb_data.proteins):
        p.pdb_ap_fn = p.suggest_ap_pdb_fn()
        p.cmm_ap_fn = ""#p.suggest_ap_cmm_fn() - do not generate cmm files
        #command=gmm_pdb_command(p.prot_fn,ape.prot_num_ap[i],p.pdb_ap_fn,p.cmm_ap_fn)
        command=gmm_pdb_command(p.prot_fn,p.num_ap,p.pdb_ap_fn,p.cmm_ap_fn)
        #command=trn_pdb_command(p.prot_fn,ape.prot_num_ap[i],p.pdb_ap_fn,p.cmm_ap_fn)
        print command
        os.system(command)
        total_num_ap = total_num_ap + p.num_ap
    #run gmm for the map
    map_d=asmb_data.map

    map_d.pdb_ap_fn=map_d.suggest_fine_ap_pdb_fn()
    map_d.cmm_ap_fn=""#map_d.suggest_ap_cmm_fn()
    #generate fine gmm
    #command = gmm_em_command(map_d.filename,ape.total_ap,float(sys.argv[2]),map_d.spacing, map_d.pdb_ap_fn,map_d.cmm_ap_fn)
    command = gmm_em_command(map_d.filename,total_num_ap,float(sys.argv[2]),map_d.spacing, map_d.pdb_fine_ap_fn,map_d.x_origin,map_d.y_origin,map_d.z_origin,map_d.cmm_ap_fn)
    #command = trn_em_command(map_d.filename,ape.total_ap,float(sys.argv[3]),map_d.spacing, map_d.pdb_ap_fn,map_d.cmm_ap_fn)
    print command
    os.system(command)
    #generate coarse gmm
    command = gmm_em_command(map_d.filename,len(asmb_data.proteins),float(sys.argv[2]),map_d.spacing, map_d.pdb_coarse_ap_fn,map_d.x_origin,map_d.y_origin,map_d.z_origin,map_d.cmm_ap_fn)
    print command
    os.system(command)



if __name__=="__main__":
    usage()
    run()
