#!/usr/bin/python

'''
TODO:
set_restraint, after the JT is updated, you should get all of the interacting pairs and set restraints only between them
'''
import sys
import IMP
import IMP.domino
import IMP.core
import IMP.atom
import IMP.helper
from assembly_input_reader import AssemblyReader, read_components,read_ref_components
from fitting_solutions_reader import read_transformations
import helpers
import probstat
import random, math
from restraint_filename_generator import create_singleton_filename, create_pairwise_filename

class run_domino:
  def __init__(self):
    self.conf={}
  def set_mapped_pair_restraints(self):
      pairs=self.parse_jt(self.jt)
      self.remove_pair_restraints()
      for r in self.set_pair_restraints(pairs):
        self.rsrs.append(r)

  def rmsd2ref(self):
    return IMP.atom.get_rmsd(self.ref_xyz,self.model_xyz)
  
  def load_map(self):
    dmap=helpers.load_map(self.asmb.map.filename,self.asmb.map.spacing,self.asmb.map.resolution,self.asmb.map.x_origin,self.asmb.map.y_origin,self.asmb.map.z_origin)
    return dmap
  
  def parse_jt(self,jt):
    pairs=[]
    for i in range(jt.get_number_of_nodes()):
      for j1 in range(jt.get_number_of_components(i)):
        for j2 in range(j1+1,jt.get_number_of_components(i)):
          pairs.append([jt.get_component_name(i,j1),
                        jt.get_component_name(i,j2)])
    return pairs
      
  def set_rigid_bodies(self):
    self.rbs = helpers.set_rigid_bodies(self.mdl,self.mhs)
    self.name_key = IMP.StringKey("name")
    self.rbs_dic={}
    for i,rb_p in enumerate(self.rbs):
        name = self.asmb.proteins[i].name
        self.rbs_dic[name]=rb_p
        rb_p.get_particle().add_attribute(self.name_key,name)

  def set_components(self,coarse=0):
      mhs = read_components(self.asmb,self.mdl)
      self.ref_mhs = read_ref_components(self.asmb,self.mdl)
      #add shell_key to the molecules leaves
      #for mh in mhs:
      #  IMP.multifit.add_surface_index(mh,3.0,1.0)
      if coarse > 0:
        self.mhs=helpers.coarse_molecules(mhs,coarse)
      else:
        self.mhs=mhs
      self.model_xyz=IMP.core.XYZs()
      for mh in self.mhs:
        self.model_xyz = self.model_xyz + IMP.core.XYZs(IMP.core.get_leaves(mh))
      self.ref_xyz=IMP.core.XYZs()
      for mh in self.ref_mhs:
        self.ref_xyz = self.ref_xyz + IMP.core.XYZs(IMP.core.get_leaves(mh))

  def set_single_restraints(self):
      dmap = self.load_map()
      rsrs=[]
      for i,mh in enumerate(self.mhs):
        ps=IMP.Particles()
        ps.append(self.rbs[i].get_particle())
        #rsrs.append([helpers.set_em_restraint(mh,dmap),ps,1.])
        rsrs.append([helpers.set_em_restraint(mh,dmap),ps,3.3])
      return rsrs

  def set_pair_restraints(self,pairs):
      #set pair restraints
      rsrs=[]
      for p in pairs:
        rb1=self.rbs_dic[p[0]]
        rb2=self.rbs_dic[p[1]]
        ps=IMP.Particles()
        ps.append(rb1.get_particle())
        ps.append(rb2.get_particle())
        rsrs.append([
           helpers.set_weighted_excluded_volume_between_rigid_bodies(rb1,rb2),
           ps,
           #1.])
           0.02])
        #TODO : add weight restraint
      #set single restraints
      #rsrs.append([self.set_em_restraint(self.mhs,dmap),self.rbs,1.])
      return rsrs


  def remove_pair_restraints(self):
      #set pair restraints
      copy_rsrs=self.rsrs
      self.rsrs=[]
      for r in copy_rsrs:
        if len(r[1])==2:
          self.mdl.remove_restraint(r[0])
        else:
          self.rsrs.append(r)
          
  def setup(self,asmb,jt,coarse=0):
      self.mdl = IMP.Model()
      self.asmb=asmb
      self.set_components(coarse)
      self.set_rigid_bodies()
      self.tu=IMP.domino.TransformationUtils(self.rbs,True) #because we need the reference to be the initial positions
      self.jt=jt
      self.rsrs=self.set_single_restraints()
      
  def setup_optimization(self,num_solutions=1):
      #print "===========start setup optimization"
      self.re=IMP.domino.RestraintEvaluatorFromFile()
      self.d_opt=IMP.domino.DominoOptimizer(self.jt,self.mdl,self.re)
      key=IMP.domino.node_name_key()
      #IMP.set_log_level(IMP.SILENT)#VERBOSE)
      for r,ps,w in self.rsrs:
        if (len(ps) == 2) : #pairwise term
          fn=create_pairwise_filename(ps[0],ps[1])
          #print "=========add restraint: " + fn
          self.re.set_restraint_file(r,fn)
          self.d_opt.add_restraint(r,ps,w)
        elif (len(ps) == 1): #singleton term
          fn=create_singleton_filename(ps[0])
          #print "=========add restraint: " + fn
          self.re.set_restraint_file(r,fn)
          self.d_opt.add_restraint(r,ps,w)
        else:
          print "ERROR - no file attached to this restraint"
          sys.exit(1)
      #self.sampler.show()
      self.d_opt.set_sampling_space(self.sampler)
      self.d_opt.set_number_of_solutions(num_solutions)

  def optimize_exhustive(self,ref_mdl):
      print "START EXH"
      combs=IMP.domino.CombStates()
      #IMP.set_log_level(IMP.VERBOSE)
      self.setup_optimization(1)
      self.d_opt.exhaustive_enumeration(combs,True,True)
      print "after exhaustive_enumeration"
      rmsd=self.evaluate_rmsd_to_reference(combs,ref_mdl)
      print "after rmsd"
      scores=IMP.domino.FloatsVec()
      self.d_opt.score_combinations(combs,scores)
      print "after scores"
      #calc all scores      
      min_comb=None
      min_score=10000
      print "Number of combinations: " + str(len(combs))
      for i,comb in enumerate(combs):
        reg_line="REG|"+comb.key()
        reg_line=reg_line+"|"+str(scores[i]).replace(",","|")[1:-1]+"|"+str(rmsd[i])+"|"
        print reg_line
        if min_score>comb.get_total_score():
          min_score =  comb.get_total_score()
          min_comb=comb
  def optimize(self,ap_map,sol_fn):
      #print "setup optimization"
      self.setup_optimization()
      #print "run optimization"
      #self.d_opt.get_graph().show()
      self.d_opt.optimize(1)
      #print "print combination"
      #print "going to get opt combination"
      l=""
      opt_comb=self.d_opt.get_graph().get_opt_combination(0)
      #print "after going to get opt combination"
      for mh_ind,mh in enumerate(self.mhs):
        rb_p=self.rbs[mh_ind].get_particle()
        trans_p=self.sampler.get_space(rb_p).get_state(opt_comb.get_state_num(rb_p))
        self.tu.move2state(rb_p,trans_p)
        #get the original transformation index
        orig_ind=trans_p.get_value(IMP.IntKey("orig_ind"))
        l=l+mh.get_particle().get_value(IMP.domino.node_name_key())+","
        l=l+str(orig_ind)+"|"
      l = l + "("+str(opt_comb.get_total_score())+")"
      self.conf[str(ap_map)]=[l,self.rmsd2ref(),sol_fn]
      self.d_opt.move_to_opt_comb(0)
      IMP.atom.write_pdb(self.mhs,sol_fn)
  '''
  Calculate all restraints and keep the values
  '''
  def evaluate_restraints(self,ref=None):
    scores=[] #scores[i] - all of the scores of the i'th restraint
    for r_num,[r,ps,w] in enumerate(self.rsrs):
      line = "working on restraint number:" + str(r_num) + " on particles:"
      for p in ps:
        line = line + p.get_name() +" "
      print line
      scores.append([ps,{}])
      #create the discrete sampling space relevant for this restraint
      discrete_spaces_lens=[]
      for rb in ps:
        discrete_spaces_lens.append(range(self.discrete_set.get_number_of_mapped_states(rb)))
      print "discrete spaces for restraint: " + str(r_num) + " are: " + str(discrete_spaces_lens)
      cc=probstat.Cartesian(discrete_spaces_lens)
      for states in cc:
        #move to state
        header=""
        comb_key=IMP.domino.CombState()
        for i,state in enumerate(states):
          rb=ps[i]
          self.tu.move2state(rb,
                        self.discrete_set.get_mapped_state(rb,state))
          comb_key.add_data_item(rb,state)
        scores[-1][1][comb_key.key()]=r.evaluate(None)
    return scores
  
  # def exhaustive_search(self):
  #   print "start exhuastive search"
  #   fitting_results=[]
  #   for rb_i,rb in enumerate(self.rbs):
  #     fitting_results.append([])
  #     for rb_state in range(self.discrete_set.get_number_of_mapped_states(rb)):
  #       self.tu.move2state(rb,
  #           self.discrete_set.get_mapped_state(rb,rb_state))
  #       score=rsrs[-len(self.rbs)+rb_i][0].unprotected_evaluate(None)*rsrs[-len(self.rbs)+rb_i][2]
  #       print score
  #       fitting_results[-1].append(score)

  #   # print fitting_results
        
  #   f=open("trace_mapped.log","w")
  #   counter=0
  #   all_combs = self.get_all_combinations()
  #   for item in all_combs:
  #   #cc=probstat.Cartesian(discrete_spaces_lens)
  #   #for i in range(500):
  #     #item=cc[random.randint(0,len(cc))]
  #     for rb_i,rb_state in enumerate(item):
  #       self.tu.move2state(self.rbs[rb_i],
  #                     self.discrete_set.get_mapped_state(self.rbs[rb_i],rb_state))
  #     #self.mdl.update_score_states()
  #     self.mdl.evaluate()
  #     f.write(str(item)+"|")
  #     sum_=0.
  #     for r,ps,w in rsrs[:-len(self.rbs)]:
  #       score=r.unprotected_evaluate(None)*w
  #       f.write(str(score)+"|")
  #       sum_ = sum_+score
  #     for rb_i,rb_state in enumerate(item):
  #       score=fitting_results[rb_i][rb_state]
  #       f.write(str(score)+"|")
  #       sum_ = sum_+score
  #     f.write(str(self.mdl.evaluate(None))+"|")
  #     f.write(str(sum_)+"|")
  #     f.write(str(self.rmsd2ref())+"|\n")
  #     IMP.atom.write_pdb(self.mhs,"mm_"+str(counter)+".pdb")
  #     counter=counter+1
  #   f.close()



  # '''
  # Calculate rmsd to reference
  # Here we make an assumption that the ref has a chain for each rigid body
  # '''
  # def evaluate_rmsd_to_reference(self,ref):
  #   #To make things effecient we calculate RMSD for each rigid body
  #   #and then combine
  #   #IMP.set_log_level(IMP.VERBOSE)
  #   ref_chain_xyz=[]
  #   for c in IMP.atom.get_by_type(ref,IMP.atom.CHAIN_TYPE):
  #     ref_chain_xyz.append(IMP.core.XYZs(IMP.atom.get_leaves(c)))
  #   model_chain_xyz=[]
  #   for mh in self.mhs:
  #       model_chain_xyz.append(IMP.core.XYZs(IMP.core.get_leaves(mh)))
  #   #calcualte rmsd for each chain
  #   chain_rmsd=[]
  #   rmsd={}
  #   num_leaves=[]

  #   for i,rb in enumerate(self.rbs):
  #     print "starting rmsd: " + str(IMP.atom.rmsd(ref_chain_xyz[i],
  #                                           model_chain_xyz[i]))
  #   for i,rb in enumerate(self.rbs):
  #     chain_rmsd.append([])
  #     num_leaves.append(len(IMP.atom.get_leaves(self.mhs[i])))
  #     for state_ind in range(self.discrete_set.get_number_of_mapped_states(rb)):
  #       self.tu.move2state(rb,
  #                     self.discrete_set.get_mapped_state(rb,state_ind))
  #       IMP.atom.write_pdb(self.mhs[i],str(i)+"_"+str(state_ind)+".pdb")
  #       rb.get_model().evaluate(None) #to make sure the rigid body position is updated

  #       chain_rmsd[-1].append(IMP.atom.rmsd(ref_chain_xyz[i],
  #                                           model_chain_xyz[i]))
  #     print "chain rmsd: " + str(chain_rmsd[-1])
  #     discrete_spaces_lens=[]
  #   for rb in self.rbs:
  #     discrete_spaces_lens.append(range(self.discrete_set.get_number_of_mapped_states(rb)))
  #   cc=probstat.Cartesian(discrete_spaces_lens)
  #   for states in cc:
  #     #move to state
  #     comb_key=IMP.domino.CombState()
  #     full_rmsd=0.0
  #     for i,state in enumerate(states):
  #       rb=self.rbs[i]
  #       comb_key.add_data_item(rb,state)
  #       #add full_rmsd calculation
  #       full_rmsd=full_rmsd+chain_rmsd[i][state]*chain_rmsd[i][state]*num_leaves[i]
  #     rmsd[comb_key.key()]=math.sqrt(full_rmsd/sum(num_leaves))
  #   return rmsd

        

  #   rmsd={} #rmsd[key] - rmsd to of permutation with key to reference
  #   discrete_spaces_lens=[]
  #   for rb in self.rbs:
  #     discrete_spaces_lens.append(range(self.discrete_set.get_number_of_mapped_states(rb)))
  #   cc=probstat.Cartesian(discrete_spaces_lens)
  #   for states in cc:
  #     #move to state
  #     comb_key=IMP.domino.CombState()
  #     for i,state in enumerate(states):
  #       rb=self.rbs[i]
  #       self.tu.move2state(rb,
  #                     self.discrete_set.get_mapped_state(rb,state))
  #       comb_key.add_data_item(rb,state)
  #     self.rbs[0].get_model().evaluate(None)#to update score states
  #     rmsd[comb_key.key()]=IMP.atom.rmsd(ref_xyz,model_xyz)
  #   return rmsd


  def get_all_combinations(self):
    # print fitting_results
    discrete_spaces_lens=[]
    for rb in self.rbs:
      rb_p=rb.get_particle()
      discrete_spaces_lens.append(range(self.discrete_set.get_number_of_mapped_states(rb_p)))
    return probstat.Cartesian(discrete_spaces_lens)

  '''
  Calculate rmsd to reference to each of the input combinations
  '''
  def evaluate_rmsd_to_reference(self,combs,ref):
    combs_rmsd=[]
    ref_xyz=IMP.core.XYZs(IMP.atom.get_leaves(ref))
    model_xyz=IMP.core.XYZs()
    for mh in self.mhs:
      for xyz in IMP.core.XYZs(IMP.core.get_leaves(mh)):
        model_xyz.append(xyz)
        
    for comb in combs:
      #move to combination
      self.sampler.move2state(comb)
      combs_rmsd.append(IMP.atom.get_rmsd(ref_xyz,
                                          model_xyz))
    return combs_rmsd

