#!/usr/bin/python

from run_domino import run_domino
import sys
import IMP
import IMP.atom
import IMP.domino
import probstat
import assembly_input_reader
import sampling_space
import mapping_engine
import calculate_pairwise_terms
def usage():
  if not (len(sys.argv) == 5 or len(sys.argv) == 4):
    print sys.argv[0] + " <assembly.input> <assembly junction tree> <configuration output file> <OPTIONAL: reference model>"
    print "Fit proteins into a cryoEM density map of their assembly"
    sys.exit()
  if len(sys.argv) == 4:
    sys.argv.append("")

#The junction tree is of the anchor points
def set_correct_mapping_in_jt(jt,orig_jt,comp2ap,asmb):
  ap2comp={}
  for comp_ind,ap_ind in enumerate(comp2ap):
    ap2comp[ap_ind]=comp_ind
  for i in range(jt.get_number_of_nodes()):
    for j1 in range(jt.get_number_of_components(i)):
      #orig_name = jt.get_component_name(i,j1)
      orig_ap_ind=int(orig_jt.get_component_name(i,j1))
      jt.set_component_name(i,j1,asmb.proteins[ap2comp[orig_ap_ind]].name)

#def run( assembly_fn,jt_fn,asmb_ap_fn,mapping,ref):
def run(asmb,jt,orig_jt,comp2ap,sol_fn,orig_ds,map_comp,asmb_ap,dist_t):
    print "Infering best configuration of mapping:",map_comp
    rd=run_domino()
    rd.setup(asmb,jt)
    #load sampling space
    [trans_ps,orig_ds,sampler] = sampling_space.load_full_discrete_set(rd.rbs,asmb)
    #set correct mapping in JT
    set_correct_mapping_in_jt(jt,orig_jt,comp2ap,asmb)
    #set the correct sampling space
    [new_ds,new_sampler]=sampling_space.set_mapped_discrete_sampling_space(orig_ds,comp2ap,asmb_ap,dist_t)
    #optimize
    rd.set_mapped_pair_restraints()
    rd.sampler=new_sampler
    rd.optimize(comp2ap,sol_fn)
    return rd.conf[rd.conf.keys()[0]]
def full_run(asmb_fn,jt_fn,output_fn,ref_fn):
  IMP.set_log_level(IMP.NONE)
  #load assembly data
  asmb=assembly_input_reader.AssemblyReader(asmb_fn)
  #load junction tree
  jt=IMP.domino.JunctionTree()
  IMP.domino.read_junction_tree(jt_fn,jt)
  orig_jt=IMP.domino.JunctionTree()
  IMP.domino.read_junction_tree(jt_fn,orig_jt)

  #load anchor points
  mdl=IMP.Model()#TODO - it is currently a different model then rbs
  ap_mh=IMP.atom.read_pdb(asmb.map.pdb_coarse_ap_fn,mdl,IMP.atom.CAlphaPDBSelector())
  asmb_ap=IMP.Particles()
  for ph in IMP.core.get_leaves(ap_mh):
    asmb_ap.append(ph.get_particle())
  num_comps=len(asmb.proteins)
  
  #Find all mappings
  d2=run_domino()
  d2.setup(asmb,jt)
  [trans_ps,orig_ds,sampler] = sampling_space.load_full_discrete_set(d2.rbs,asmb)
  dist_t=8#TODO - change to resolution*2
  maps_engine=mapping_engine.Mapping(asmb,d2.mhs,asmb_ap,orig_ds,dist_t)
  comp2ap=maps_engine.build_mappings_from_components_to_anchor_points()
  print "Found " , len(comp2ap) , " valid mappings from components to anchor points"
  print "number of mapping of components to anchor points:"+str(len(comp2ap))
  for mp in comp2ap:
   print "mapping: "+ str(mp)
  #load reference structure
  if ref_fn != "":
    mhs_ref = assembly_input_reader.read_ref_components(asmb,mdl)
    ref_mdl=IMP.atom.read_pdb(ref_fn,mdl,IMP.atom.NonWaterPDBSelector())
    ref_xyz=IMP.core.XYZs()
    for mh in mhs_ref:
      ref_xyz = ref_xyz+IMP.core.XYZs(IMP.core.get_leaves(mh))
  else:
    ref_xyz=None

  #load domino optimizer
  sols=[]
  for i,map_comp in enumerate(comp2ap):
    ll=run(asmb,jt,orig_jt,map_comp,"conf."+str(i)+".pdb",orig_ds,map_comp,asmb_ap,dist_t)
    sols.append(ll)
  f=open(output_fn,"w")
  for conf_ind,i in enumerate(sols):
    l=""
    l = i[0]
    l = l + "(rmsd:"+str(i[1]) + ")"
    l = l + "("+str(i[2]) + ")"
    print l
    f.write(l+"\n")
  f.close()    
  print "All done!"



if __name__ == '__main__':
  usage()
  asmb_fn=sys.argv[1]
  jt_fn=sys.argv[2]
  output_fn=sys.argv[3]
  ref_fn=sys.argv[4]
  full_run(asmb_fn,jt_fn,output_fn,ref_fn)
