#!/usr/bin/python

'''
This script estimates the number of anchor points that should be calculate for each protein and for the density map and runs anchor point detection
'''
import sys
import IMP
import IMP.atom
from assembly_input_reader import *
from anchor_point_estimator import *
def usage():
    if len(sys.argv) != 3:
        print sys.argv[0] + " <assembly.input> <param filename>"
        sys.exit()

def fitting_command(em_fn,pdb_fn,apix,resolution,c_em_ap_fn,
                    f_em_ap_fn,pdb_ap_fn,
                    param_fn,orig=None,ref_fn="",output_fn="",
                    local_rad=-1):
    command = os.path.expandvars("/opt/multifit")+"/bin/anchor_point_fitting "+ em_fn + " " + str(apix)+" " + str(resolution) + " " + pdb_fn + " " + c_em_ap_fn + " " + f_em_ap_fn + " " + pdb_ap_fn + " --param-file " + param_fn
    if ref_fn != "":
        command=command + " --ref " + ref_fn
    if output_fn != "":
        command = command + " --output-file " + output_fn
    command = command + " --sol fits/" + pdb_fn.split("/")[-1].split(".pdb")[0]+"_fit_sol"
    #command = command + " --sol " + pdb_fn.split(".pdb")[0]+"_fit_sol"
    if not local_rad == -1:
        command = command + " --local-rad " + str(local_rad)
    if not orig==None:
        command = command + " --x " + str(orig[0])
        command = command + " --y " + str(orig[1])
        command = command + " --z " + str(orig[2])
    return command

def run():
    asmb_data = AssemblyReader(sys.argv[1])
    map_d=asmb_data.map
    param_fn = sys.argv[2]
    orig=IMP.algebra.Vector3D(map_d.x_origin,map_d.y_origin,map_d.z_origin)
    fits_dir="./fits"
    try:
        os.mkdir(fits_dir)
    except:
        None #directory exists
    for i,p in enumerate(asmb_data.proteins):
        command = fitting_command(map_d.filename,p.prot_fn,
                                  map_d.spacing, map_d.resolution,
                                  map_d.pdb_coarse_ap_fn,
                                  map_d.pdb_fine_ap_fn,p.pdb_ap_fn,
                                  param_fn,orig,p.ref_fn,p.transformations_fn)
        print command
        os.system(command)

if __name__=="__main__":
    usage()
    run()
